"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = require("react");
var rsup_duration_1 = require("rsup-duration");
var orderFinally_1 = require("./orderFinally");
var SingleWaiter = /** @class */ (function () {
    function SingleWaiter() {
        this._count = 0;
        this._listeners = [];
        this.wait = this.wait.bind(this);
        this.useWait = this.useWait.bind(this);
    }
    Object.defineProperty(SingleWaiter.prototype, "isInUse", {
        get: function () {
            return this.isWaiting || this._listeners.length > 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SingleWaiter.prototype, "isWaiting", {
        get: function () {
            return this._count > 0;
        },
        enumerable: true,
        configurable: true
    });
    SingleWaiter.prototype._emit = function () {
        this._listeners.forEach(function (fn) { return fn(); });
    };
    SingleWaiter.prototype._subscribe = function (listener) {
        var _this = this;
        this._listeners.push(listener);
        return function () {
            _this._listeners.splice(_this._listeners.indexOf(listener), 1);
        };
    };
    SingleWaiter.prototype.wait = function (order) {
        var _this = this;
        if (++this._count === 1)
            this._emit();
        return orderFinally_1.default(order, function () {
            if (--_this._count === 0)
                _this._emit();
        });
    };
    // tslint:disable-next-line: cognitive-complexity
    SingleWaiter.prototype.useWait = function (_a) {
        var _this = this;
        var _b = _a === void 0 ? {} : _a, _c = _b.delay, delay = _c === void 0 ? 0 : _c, _d = _b.duration, duration = _d === void 0 ? 0 : _d;
        var _e = react_1.useState(this.isWaiting), isWaiting = _e[0], setWaiting = _e[1];
        var memoUnSubscriber = react_1.useMemo(function () {
            var delayer = delay > 0 ? rsup_duration_1.default(delay) : null;
            var persister = duration > 0 ? rsup_duration_1.default(duration) : null;
            var next = null;
            var unmounted = false;
            if (isWaiting && persister) {
                persister.start().then(afterDuration);
            }
            var unsubscribe = _this._subscribe(function () {
                if ((delayer && delayer.isDuring) || (persister && persister.isDuring)) {
                    next = _this.isWaiting;
                    return;
                }
                if (_this.isWaiting) {
                    if (delayer) {
                        next = true;
                        delayer.start().then(afterDelay);
                    }
                    else {
                        startWaiting();
                    }
                }
                else {
                    setWaiting(false);
                }
            });
            return function () {
                unmounted = true;
                unsubscribe();
            };
            function startWaiting() {
                setWaiting(true);
                if (persister)
                    persister.start().then(afterDuration);
            }
            function afterDelay() {
                if (unmounted)
                    return;
                if (next === true)
                    startWaiting();
                next = null;
            }
            function afterDuration() {
                if (unmounted)
                    return;
                if (next === false)
                    setWaiting(false);
                next = null;
            }
        }, [delay, duration]);
        react_1.useLayoutEffect(function () { return memoUnSubscriber; }, [memoUnSubscriber]);
        return isWaiting;
    };
    return SingleWaiter;
}());
exports.default = SingleWaiter;
